@@ -4,7 +4,7 @@ require 'fileutils'
 $CFLAGS << ' -std=c99 -D_POSIX_C_SOURCE=200809L -Wall -Wextra'
 
 if enable_config('fail-on-warning')
-  $CFLAGS << ' -Werrno'
+  $CFLAGS << ' -Werror'
 end
 
 if enable_config('debug')
