@@ -409,6 +409,15 @@ VALUE mm_unmap(VALUE obj) {
             }
             free(i_mm->t->path);
         }
+
+        // Ensure any lingering RString values get a length of zero. We
+        // can't zero out the address since GET_MMAP() inside
+        // mm_update_obj_i() expects a non-null address and path.
+        i_mm->t->len = 0;
+        i_mm->t->real = 0;
+        mm_update(obj);
+
+        i_mm->t->addr = NULL;
         i_mm->t->path = NULL;
     }
 
