@@ -66,7 +66,7 @@ static int perform_mmap(mm_ipc *i_mm, size_t len) {
     return SUCCESS;
 }
 
-static int expand(mm_ipc *i_mm, size_t len) {
+static int expand(VALUE self, mm_ipc *i_mm, size_t len) {
     if (len < i_mm->t->len) {
         return with_exception(rb_eArgError, "Can't reduce the size of mmap");
     }
@@ -88,6 +88,8 @@ static int expand(mm_ipc *i_mm, size_t len) {
         return with_exception_errno(rb_eArgError, "mlock(%d)", errno);
     }
 
+    mm_update(self);
+
     return SUCCESS;
 }
 
@@ -151,7 +153,7 @@ uint32_t load_used(mm_ipc *i_mm) {
 
 void save_used(mm_ipc *i_mm, uint32_t used) { *((uint32_t *)i_mm->t->addr) = used; }
 
-static VALUE initialize_entry(mm_ipc *i_mm, VALUE positions, VALUE key, VALUE value) {
+static VALUE initialize_entry(VALUE self, mm_ipc *i_mm, VALUE positions, VALUE key, VALUE value) {
     if (i_mm->t->flag & MM_FROZEN) {
         rb_error_frozen("mmap");
     }
@@ -166,10 +168,11 @@ static VALUE initialize_entry(mm_ipc *i_mm, VALUE positions, VALUE key, VALUE va
 
     uint32_t used = load_used(i_mm);
     while (i_mm->t->len < (used + entry_length)) {
-        if (!expand(i_mm, i_mm->t->len * 2)) {
+        if (!expand(self, i_mm, i_mm->t->len * 2)) {
             raise_last_exception();
         }
     }
+
     save_entry(i_mm, used, key, value);
     save_used(i_mm, used + entry_length);
 
@@ -189,7 +192,7 @@ VALUE method_fetch_entry(VALUE self, VALUE positions, VALUE key, VALUE default_v
         return load_value(i_mm, position);
     }
 
-    position = initialize_entry(i_mm, positions, key, default_value);
+    position = initialize_entry(self, i_mm, positions, key, default_value);
     return load_value(i_mm, position);
 }
 
@@ -207,7 +210,7 @@ VALUE method_upsert_entry(VALUE self, VALUE positions, VALUE key, VALUE value) {
         return load_value(i_mm, position);
     }
 
-    position = initialize_entry(i_mm, positions, key, value);
+    position = initialize_entry(self, i_mm, positions, key, value);
     return load_value(i_mm, position);
 }
 
@@ -229,7 +232,7 @@ VALUE method_save_used(VALUE self, VALUE value) {
     }
 
     if (i_mm->t->len < INITIAL_SIZE) {
-        if (!expand(i_mm, INITIAL_SIZE)) {
+        if (!expand(self, i_mm, INITIAL_SIZE)) {
             raise_last_exception();
         }
     }
